import java.awt.*;
import java.net.*;

/**
 * Lottorobics is an applet that conducts a "workout" of six-
 * number lottery numbers ranging from 1 to 50. The user picks six
 * favorite numbers (or uses quick-pick to generate six at random),
 * and sees how many times those numbers win in an escalating number
 * of drawings. Copyright 2001 Rogers Cadenhead. You can use this
 * applet freely on your own web pages if you indicate (1) the
 * author of the applet and (2) a link to the
 * <a href="http://www.cadenhead.org/projects/misc/lottorobics.shtml">Lottorobics</a>
 * home page on the same page where you run the applet. This
 * source code cannot be redistributed or published without the
 * express consent of Rogers Cadenhead.
 * @author <a href="mailto:rogers@cadenhead.org">Rogers Cadenhead</a>
 * @version 1.1
 */
public class Lottorobics extends java.applet.Applet
    implements Runnable {

    Thread playing;
    Image background;
    Color darkGreen = new Color(6, 56, 4);
    Color lightGreen = new Color(54, 95, 51);
    Color backColor;
    
    // set up row 1
    Panel row1 = new Panel();
    CheckboxGroup option = new CheckboxGroup();
    Checkbox quickpick = new Checkbox("Quick Pick", option, false);
    Checkbox personal = new Checkbox("Personal", option, true);
    // set up row 2
    Panel row2 = new Panel();
    Label numbersLabel = new Label("Your picks: ", Label.RIGHT);
    TextField[] numbers = new TextField[6];
    Label winnersLabel = new Label("Winners: ", Label.RIGHT);
    TextField[] winners = new TextField[6];
    // set up row 3
    Panel row3 = new Panel();
    Button stop = new Button("Stop");
    Button play = new Button("Play");
    Button reset = new Button("Reset");
    // set up row 4
    Panel row4 = new Panel();
    Label got3Label = new Label("3 of 6: ", Label.RIGHT);
    TextField got3 = new TextField("0", 6);
    Label got4Label = new Label("4 of 6: ", Label.RIGHT);
    TextField got4 = new TextField("0", 6);
    Label got5Label = new Label("5 of 6: ", Label.RIGHT);
    TextField got5 = new TextField("0", 6);
    Label got6Label = new Label("6 of 6: ", Label.RIGHT);
    TextField got6 = new TextField("0", 6);
    Label drawingsLabel = new Label("Drawings: ", Label.RIGHT);
    TextField drawings = new TextField("0", 6);
    Label yearsLabel = new Label("Years: ", Label.RIGHT);
    TextField years = new TextField("0", 6);

    public void init() {
        if (System.getProperty("java.vendor").indexOf("Microsoft") > -1)
            backColor = lightGreen;
        else
            backColor = darkGreen;
        System.out.println("Lottorobics version 1.1\n" +
            "Copyright 1998 Rogers Cadenhead\n" +
            "http://www.prefect.com/java24/lottorobics.html");
        setBackground(backColor);
        background = getImage(getCodeBase(), "lottoback.jpg");
        Font f = new Font("Helvetica", Font.BOLD, 13);
        Insets in = new Insets(10, 10, 10, 10);
        GridBagLayout appletLayout = new GridBagLayout();
        setLayout(appletLayout);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 1;
        gbc.fill = GridBagConstraints.NONE;
        gbc.ipadx = 5;
        gbc.ipady = 0;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.insets = in;

        FlowLayout layout1 = new FlowLayout(FlowLayout.CENTER, 10, 10);
        row1.setBackground(backColor);
        row1.setLayout(layout1);
        quickpick.setForeground(Color.green);
        quickpick.setFont(f);
        row1.add(quickpick);
        personal.setForeground(Color.green);
        personal.setFont(f);
        row1.add(personal);
        appletLayout.setConstraints(row1, gbc);
        add(row1);

        GridBagLayout layout2 = new GridBagLayout();
        GridBagConstraints gbc2 = new GridBagConstraints();
        Insets in2 = new Insets(2, 2, 2, 2);
        gbc2.gridx = 1;
        gbc2.gridy = 1;
        gbc2.fill = GridBagConstraints.VERTICAL;
        gbc2.ipadx = 5;
        gbc2.ipady = 0;
        gbc2.anchor = GridBagConstraints.EAST;
        gbc2.insets = in2;
        row2.setBackground(backColor);
        row2.setLayout(layout2);
        numbersLabel.setForeground(Color.green);
        numbersLabel.setFont(f);
        gbc2.gridx = 1;
        gbc2.gridy = 1;
        layout2.setConstraints(numbersLabel, gbc2);
        row2.add(numbersLabel);
        gbc2.anchor = GridBagConstraints.CENTER;
        gbc2.weightx = 2;
        gbc2.weightx = 1;
        for (int i = 0; i < 6; i++) {
            numbers[i] = new TextField(3);
            numbers[i].setBackground(Color.white);
            gbc2.gridx = 2 + i;
            gbc2.gridy = 1;
            layout2.setConstraints(numbers[i], gbc2);
            row2.add(numbers[i]);
        }
        gbc2.anchor = GridBagConstraints.EAST;
        gbc2.weightx = 2;
        gbc2.gridx = 1;
        gbc2.gridy = 2;
        winnersLabel.setForeground(Color.green);
        winnersLabel.setFont(f);
        layout2.setConstraints(winnersLabel, gbc2);
        row2.add(winnersLabel);
        gbc2.anchor = GridBagConstraints.CENTER;
        gbc2.weightx = 1;
        for (int i = 0; i < 6; i++) {
            winners[i] = new TextField(3);
            winners[i].setBackground(Color.white);
            winners[i].setEditable(false);
            gbc2.gridx = 2 + i;
            gbc2.gridy = 2;
            layout2.setConstraints(winners[i], gbc2);
            row2.add(winners[i]);
        }
        gbc.gridx = 1;
        gbc.gridy = 2;
        appletLayout.setConstraints(row2, gbc);
        add(row2);

        FlowLayout layout3 = new FlowLayout(FlowLayout.CENTER, 5, 5);
        row3.setBackground(backColor);
        row3.setLayout(layout3);
        stop.enable(false);
        reset.enable(false);
        stop.setFont(f);
        stop.setForeground(Color.black);
        play.setFont(f);
        play.setForeground(Color.black);
        reset.setFont(f);
        reset.setForeground(Color.black);
        row3.add(stop);
        row3.add(play);
        row3.add(reset);
        gbc.gridx = 1;
        gbc.gridy = 3;
        appletLayout.setConstraints(row3, gbc);
        add(row3);

        GridBagLayout layout4 = new GridBagLayout();
        GridBagConstraints gbc3 = new GridBagConstraints();
        Insets in3 = new Insets(2, 2, 2, 2);
        gbc3.gridx = 1;
        gbc3.gridy = 1;
        gbc3.fill = GridBagConstraints.VERTICAL;
        gbc3.ipadx = 5;
        gbc3.ipady = 0;
        gbc3.anchor = GridBagConstraints.EAST;
        gbc3.insets = in3;
        gbc3.weightx = 1;
        row4.setBackground(backColor);
        row4.setLayout(layout4);
        got3Label.setForeground(Color.green);
        got3Label.setFont(f);
        layout4.setConstraints(got3Label, gbc3);
        row4.add(got3Label);
        got3.setEditable(false);
        got3.setBackground(Color.white);
        gbc3.anchor = GridBagConstraints.CENTER;
        gbc3.gridx = 2;
        gbc3.gridy = 1;
        layout4.setConstraints(got3, gbc3);
        row4.add(got3);
        got4Label.setForeground(Color.green);
        got4Label.setFont(f);
        gbc3.anchor = GridBagConstraints.EAST;
        gbc3.gridx = 3;
        gbc3.gridy = 1;
        layout4.setConstraints(got4Label, gbc3);
        row4.add(got4Label);
        got4.setEditable(false);
        got4.setBackground(Color.white);
        gbc3.anchor = GridBagConstraints.CENTER;
        gbc3.gridx = 4;
        gbc3.gridy = 1;
        layout4.setConstraints(got4, gbc3);
        row4.add(got4);
        got5Label.setForeground(Color.green);
        got5Label.setFont(f);
        gbc3.anchor = GridBagConstraints.EAST;
        gbc3.gridx = 5;
        gbc3.gridy = 1;
        layout4.setConstraints(got5Label, gbc3);
        row4.add(got5Label);
        got5.setEditable(false);
        got5.setBackground(Color.white);
        gbc3.anchor = GridBagConstraints.CENTER;
        gbc3.gridx = 6;
        gbc3.gridy = 1;
        layout4.setConstraints(got5, gbc3);
        row4.add(got5);
        got6Label.setForeground(Color.green);
        got6Label.setFont(f);
        gbc3.anchor = GridBagConstraints.EAST;
        gbc3.gridx = 1;
        gbc3.gridy = 2;
        layout4.setConstraints(got6Label, gbc3);
        row4.add(got6Label);
        got6.setEditable(false);
        got6.setBackground(Color.white);
        gbc3.anchor = GridBagConstraints.CENTER;
        gbc3.gridx = 2;
        gbc3.gridy = 2;
        layout4.setConstraints(got6, gbc3);
        row4.add(got6);
        drawingsLabel.setForeground(Color.green);
        drawingsLabel.setFont(f);
        gbc3.anchor = GridBagConstraints.EAST;
        gbc3.gridx = 3;
        gbc3.gridy = 2;
        layout4.setConstraints(drawingsLabel, gbc3);
        row4.add(drawingsLabel);
        drawings.setEditable(false);
        drawings.setBackground(Color.white);
        gbc3.anchor = GridBagConstraints.CENTER;
        gbc3.gridx = 4;
        gbc3.gridy = 2;
        layout4.setConstraints(drawings, gbc3);
        row4.add(drawings);
        yearsLabel.setForeground(Color.green);
        yearsLabel.setFont(f);
        gbc3.anchor = GridBagConstraints.EAST;
        gbc3.gridx = 5;
        gbc3.gridy = 2;
        layout4.setConstraints(yearsLabel, gbc3);
        row4.add(yearsLabel);
        years.setEditable(false);
        years.setBackground(Color.white);
        gbc3.anchor = GridBagConstraints.CENTER;
        gbc3.gridx = 6;
        gbc3.gridy = 2;
        layout4.setConstraints(years, gbc3);
        row4.add(years);
        gbc.gridx = 1;
        gbc.gridy = 4;
        appletLayout.setConstraints(row4, gbc);
        add(row4);
        repaint();
    }

    public void paint(Graphics g) {
        g.drawImage(background, 0, 0, this);
    }

    public void update(Graphics g) {
        paint(g);
    }

    public boolean action(Event event, Object object) {
        if (event.target instanceof Button) {
            if (event.target.equals(reset)) {
                clearAllFields();
                reset.enable(false);
                play.enable(true);
                quickpick.enable(true);
                personal.enable(true);
                for (int i = 0; i < 6; i++) {
                    numbers[i].setEditable(true);
                }
            }
            if (event.target.equals(play)) {
                playing = new Thread(this);
                play.enable(false);
                stop.enable(true);
                reset.enable(false);
                quickpick.enable(false);
                personal.enable(false);
                playing.start();                
            }
            if (event.target.equals(stop)) {
                stopLotto(false);
            }
        }
        if (event.target instanceof Checkbox) {
            if (event.target.equals(quickpick)) {
                for (int i = 0; i < 6; i++) {
                    int pick;
                    do {
                        pick = (int)Math.floor(Math.random() * 50 + 1);
                    } while (numberGone(pick, numbers, i));
                    numbers[i].setText("" + pick);
                }
            } else {
                for (int i = 0; i < 6; i++)
                    numbers[i].setText("   ");
            }
        }
        return true;
    }

    void stopLotto(boolean bigWinner) {
        play.enable(true);
        stop.enable(false);
        reset.enable(true);
        quickpick.enable(true);
        personal.enable(true);
        for (int i = 0; i < 6; i++) {
            numbers[i].setEditable(false);
        }
        while (playing != null) {
            playing = null;
        }
    }

    void clearAllFields() {
        for (int i = 0; i < 6; i++) {
            numbers[i].setText("");
            winners[i].setText("");
        }
        got3.setText("0");
        got4.setText("0");
        got5.setText("0");
        got6.setText("0");
        drawings.setText("0");
        years.setText("0");
    }

    void addOneToField(TextField field) {
        int num = Integer.parseInt(field.getText());
        num++;
        field.setText("" + num);
    }

    public void stop() {
        while (playing != null) {
            stop.enable(false);
            reset.enable(false);
            play.enable(true);
            quickpick.enable(true);
            personal.enable(true);
            playing = null;
            clearAllFields();
        }
    }

    boolean numberGone(int num, TextField[] pastNums, int count) {
        for (int i = 0; i < count; i++)
            if (Integer.parseInt(pastNums[i].getText()) == num)
                return true;
        return false;
    }

    boolean matchedOne(TextField win, TextField[] allPicks) {
        for (int i = 0; i < 6; i++) {
            String winText = win.getText();
            if ( winText.equals( allPicks[i].getText() ) )
                return true;
        }
        return false;
    }

    public void run() {
        Thread thisThread = Thread.currentThread();
        while (playing == thisThread) {
            addOneToField(drawings);
            int draw = Integer.parseInt(drawings.getText());
            float numYears = (float)draw / 104;
            years.setText("" + numYears);

            int matches = 0;
            for (int i = 0; i < 6; i++) {
                int ball;
                do {
                    ball = (int)Math.floor(Math.random() * 50 + 1);
                } while (numberGone(ball, winners, i));
                winners[i].setText("" + ball);
                if (matchedOne(winners[i], numbers))
                    matches++;
            }
            switch (matches) {
                case 3:
                    addOneToField(got3);
                    break;
                case 4:
                    addOneToField(got4);
                    break;
                case 5:
                    addOneToField(got5);
                    break;
                case 6:
                    addOneToField(got6);
                    stopLotto(true);
            }
            try { Thread.sleep(200); }
            catch (InterruptedException e) {}
        }
    }
}

